#ifndef TILE_ITEM_H
#define TILE_ITEM_H

#include <QQuickItem>
#include <QSGNode>
#include <QtQuick/qsgflatcolormaterial.h>

// --- QtQuick item ---

class TileItem : public QQuickItem {
  Q_OBJECT
  Q_PROPERTY(qreal size READ size WRITE setSize NOTIFY sizeChanged FINAL)

 public:
  TileItem(QQuickItem* parent=0);

  qreal size() const;
  void setSize(qreal size);

 Q_SIGNALS:
  void sizeChanged();

 protected:
  virtual QSGNode* updatePaintNode(QSGNode*, UpdatePaintNodeData*);

 private:
  qreal size_;

  Q_DISABLE_COPY(TileItem)
};

// --- Scene graph material ---

class TileMaterial : public QSGMaterial {
 public:
  TileMaterial();

  virtual QSGMaterialType* type() const;
  virtual QSGMaterialShader* createShader() const;

  void setColor(const QColor& color);
  const QColor& color() const { return color_; }

  void setScale(float scale);
  const float scale() const { return scale_; }

  int compare(const QSGMaterial* other) const;

private:
    QColor color_;
    float scale_;
};

// --- Scene graph material shader ---

class TileShader : public QSGMaterialShader {
 public:
  virtual void updateState(
      const RenderState& state, QSGMaterial* newEffect, QSGMaterial* oldEffect);
  virtual char const* const* attributeNames() const;

  static QSGMaterialType type;

 private:
  virtual void initialize();
  virtual const char* vertexShader() const;
  virtual const char* fragmentShader() const;

  int combined_id_;
  int modelview_id_;
  int color_id_;
  int scale_id_;
};

// --- Scene graph node ---

class TileNode : public QSGGeometryNode {
 public:
  TileNode(TileItem* item);
  virtual ~TileNode();

  void update();
  void preprocess();
  void setSize(float size);

 private:
  void updateGeometry();

  TileItem* item_;
  QSGGeometry geometry_;
  TileMaterial material_;
  float size_;
  bool dirty_geometry_;
};

QML_DECLARE_TYPE(TileItem)

#endif  // TILE_ITEM_H
